/* ************************************************************************** */
/*                                                                            */
/*                                                        :::      ::::::::   */
/*   libft.h                                            :+:      :+:    :+:   */
/*                                                    +:+ +:+         +:+     */
/*   By: gbersac <gbersac@student.42.fr>            +#+  +:+       +#+        */
/*                                                +#+#+#+#+#+   +#+           */
/*   Created: 2013/11/30 12:40:04 by gbersac           #+#    #+#             */
/*   Updated: 2013/12/03 17:40:46 by gbersac          ###   ########.fr       */
/*                                                                            */
/* ************************************************************************** */

#ifndef LIBFT_H
# define LIBFT_H

#include <stdlib.h>
#include <unistd.h>

typedef unsigned char t_uchar;

/******************/
/*MEMORY FUNCTIONS*/
/******************/
void		*ft_memset(void *, int, size_t);
void		ft_bzero(void *, size_t);
void		*ft_memcpy(void *, const void *, size_t);
void		*ft_memccpy(void *, const void *, int, size_t);
void		*ft_memmove(void *, const void *, size_t);
void		*ft_memchr(const void *, int, size_t);

/*segfault if NULL arg*/
int			ft_memcmp(const void *, const void *, size_t);
void		*ft_memalloc(size_t);
void		ft_memdel(void **);

/********************/
/*STR INIT FUNCTIONS*/
/********************/
char		*ft_strdup(const char *);
char		*ft_strmap(char const *s, char (*f)(char));
char		*ft_strmapi(char const *s, char (*f)(unsigned int, char));
char		*ft_strnew(size_t size);
void		ft_strdel(char **as);

/* Function which allocate a new string and fill it with the concat of
** the two strings in param and return it.
*/
char		*ft_strcup(char *, const char *);

/**************************/
/*STR COMPARISON FUNCTIONS*/
/**************************/
char		*ft_strchr(const char *s, int c);
char		*ft_strrchr(const char *s, int c);
char		*ft_strstr(const char *, const char *);
char		*ft_strnstr(const char *, const char *, size_t);
int			ft_strcmp(const char *, const char *);
int			ft_strncmp(const char *, const char *, unsigned int);

/*segfault if null arg*/
int			ft_strequ(char const *s1, char const *s2);
/*segfault if null arg*/
int			ft_strnequ(char const *s1, char const *s2, size_t n);

/********************/
/*STR COPY FUNCTIONS*/
/********************/
char		*ft_strcpy(char *, const char *);
char		*ft_strncpy(char *, const char *, size_t);
char		*ft_strcat(char *, const char *);
char		*ft_strncat(char *, const char *, size_t);
size_t		ft_strlcat(char *, const char *, size_t);

/************/
/*STR OTHERS*/
/************/
size_t		ft_strlen(const char *);
void		ft_strclr(char *s);
void		ft_striter(char *s, void (*f)(char *));
void		ft_striteri(char *s, void (*f)(unsigned int, char *));
char		*ft_strsub(char const *s, unsigned int start, size_t len);
char		*ft_strjoin(char const *s1, char const *s2);
char		**ft_strsplit(char const *s, char c);
char		*ft_itoa(int n);
/*segfault if arg is NULL*/
int			ft_atoi(const char *);
int			ft_toupper(int);
int			ft_tolower(int);
char		*ft_strtrim(char const *s);

/****************/
/*TEST FUNCTIONS*/
/****************/
int			ft_isalpha(int);
int			ft_isdigit(int);
int			ft_isalnum(int);
int			ft_isascii(int);
int			ft_isprint(int);

/*******************/
/*DISPLAY FUNCTIONS*/
/*******************/
void		ft_putchar(char c);
void		ft_putstr(char const *s);
void		ft_putendl(char const *s);
void		ft_putnbr(int n);
void		ft_putchar_fd(char c, int fd);
void		ft_putstr_fd(char const *s, int fd);
void		ft_putendl_fd(char const *s, int fd);
void		ft_putnbr_fd(int n, int fd);

/****************/
/*LIST FUNCTIONS*/
/****************/
typedef struct	s_list
{
	void			*content;
	size_t			content_size;
	struct s_list	*next;
}				t_list;

/* Allocate a new node and fill it with a copy of content which size is
** content_size.
*/
t_list		*ft_lstnew(void const *content, size_t content_size);

/* Free the node pointed by alst and all of its successors*/
void		ft_lstdel(t_list **alst, void (*del)(void *, size_t));

/*Function which add a node the beginning of the list*/
void		ft_lstadd(t_list **alst, t_list *new);

/* Function which add a node at the end of the list*/
void		ft_lstaddq(t_list **alst, t_list *new);

/* Append l2 at the end of l1.
** @return l1 or NULL if l1 pr l2 is NULL.
*/
t_list		*ft_lstmerge(t_list *l1, t_list *l2);

/* Create a new list by applying the f function on each of the node of
** the lst list.
*/
t_list		*ft_lstmap(t_list *lst, t_list * (*f)(t_list *elem));

size_t		ft_lstlen(t_list *lst);

/* Delete the node and its content. The content is deleted using the
** del function.
*/
void		ft_lstdelone(t_list **alst, void (*del)(void *, size_t));

/*Apply the f function on each of the node of lst*/
void		ft_lstiter(t_list *lst, void (*f)(t_list *elem));

/*Return the last element of the list*/
t_list		*ft_lst_last(t_list *lst);

/*Split the str string according to the c char and create a list
*/
t_list		*ft_strsplit_lst(const char *str, char separator);

/* Display all the str in the list on the terminal.
** Warning : all content of each node are supposed to be string !
*/
void		ft_putlststr(t_list *lst, const char *separator);

/* Display all the str in the list on the file fd.
** Warning : all content of each node are supposed to be string !
*/
void		ft_putlststr_fd(t_list *lst, int fd, const char *separator);

/* Return one new string which is the concatenation of all the content of
** all the string in the list separate by the separator string.
*/
char		*ft_lststr_merge(t_list *lst, const char *separator);

/* Sort the list using the merge sort algorithm.
** @lst	list to sort
** @cmp	function which return true if element need to be inserted in the lst
** 		before this node.
*/
t_list		*ft_mergesort(t_list *, t_uchar (*)(t_list *, t_list *));

/****************/
/*TREE FUNCTIONS*/
/****************/
typedef struct	s_node
{
	struct s_node	*parent;
	struct s_node	*son; /*son which is on the left*/
	struct s_node	*brother; /*to the right*/
	void			*content;
	size_t			content_size;
}				t_node;

t_node		*ft_nodenew(void const *content, size_t content_size);
int			ft_nodedelone(t_node **anode, void (*del)(void *, size_t));

/* Delete the root and all the son of anode but not their content.
*/
void		ft_nodedel(t_node **anode);
void		ft_nodeaddson(t_node *parent, t_node *new);

#endif
